#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 13 11:46:01 2023

@author: jaspermittag
"""

import random
import numpy as np
import pygame
import matplotlib.pyplot as plt

pygame.init()

WIDTH, HEIGHT = 700, 400
WIN = pygame.display.set_mode((WIDTH, HEIGHT))
pygame.display.set_caption("Brownsche Bewegung")

RED = (255, 82, 32)
BLACK = (0, 0, 0)
BLUE = (23, 2, 255)
WHITE = (255, 255, 255)
GREEN = (50, 205, 50)
YELLOW = (255, 255, 0)

FPS = 60

ballAmount = 100
ballRadius = 3
ballColor = BLUE
ballMasse = 10
SPEED = 2

BROWNSCHES_TEILCHEN_COLOR = RED
BROWNSCHES_TEILCHEN_RADIUS = 10
BROSCNHES_TEILCHEN_MASSE = 50



class Teilchen:
    def __init__(self, color, radius, masse):
        self.color = color
        self.radius = radius
        # zufällige Postion zuordnen
        self.x = random.randint(radius, WIDTH - radius)
        self.y = random.randint(radius, HEIGHT - radius)
        self.xy = np.array([self.x, self.y])
        self.vxy = np.array([random.uniform(-1, 1), random.uniform(-1, 1)])
        norm = np.sqrt(self.vxy[0]**2 + self.vxy[1]**2)
        self.vxy = self.vxy / norm
        self.masse = masse
        self.speed = SPEED
        
    def draw(self, win):
        '''zeichnet das Teilchen ins Fenster'''
        pygame.draw.circle(win, self.color, (self.x, self.y), self.radius)
        
    def move(self):
        self.x += self.vxy[0] * self.speed
        self.y += self.vxy[1] * self.speed
      
    # Kollision mit einer Wand     
    def borderCollision(self):
        if self.x >= WIDTH - self.radius:
          self.x = WIDTH - self.radius
          self.vxy[0] *= -1
        elif self.x <= self.radius:
          self.x = self.radius
          self.vxy[0] *= -1
        if self.y >= HEIGHT - self.radius:
          self.y = HEIGHT - self.radius
          self.vxy[1] *= -1
          self.speed += 0.5
          if self.speed >= 5:
              self.speed = 5
        elif self.y <= self.radius:
          self.y = self.radius
          self.vxy[1] *= -1
          self.speed -= 0.5
          if self.speed <= 0.5:
              self.speed = 0.5
         
    
    def ballCollision(self, ball2):
        x_b1 = self.x
        y_b1 = self.y
        x_b2 = ball2.x
        y_b2 = ball2.y
        
        sv_x = ball2.x - self.x
        sv_y = ball2.y - self.y
        distance = np.sqrt(sv_x**2 + sv_y**2)
        
        if distance <= self.radius + ball2.radius:
            vxy_b1 = self.vxy
            vxy_b2 = ball2.vxy
            
            self.x = x_b1
            self.y = y_b1
            ball2.x = x_b2
            ball2.y = y_b2
            
            self.vxy = (self.masse * vxy_b1 + ball2.masse *
                         (2 * vxy_b2 - vxy_b1)) / (self.masse + ball2.masse)
            ball2.vxy = (ball2.masse * vxy_b2 + self.masse *
                         (2 * vxy_b1 - vxy_b2)) / (self.masse + ball2.masse)
        
         
        
def draw(win, balls):
    win.fill(BLACK)
    for ball in balls:
        ball.draw(win)
    pygame.display.update()
    
                  

def main(): 
    run = 1
    
    # Teilchenobjekte in eine Liste schreiben
    balls = []
    for i in range(ballAmount):
        ball = "ball"+str(i)
        ball = Teilchen(ballColor, ballRadius, ballMasse)
        balls.append(ball)
    
    brownschesTeilchen = Teilchen(BROWNSCHES_TEILCHEN_COLOR, BROWNSCHES_TEILCHEN_RADIUS, 
                                  BROSCNHES_TEILCHEN_MASSE)
    balls.append(brownschesTeilchen)
    
    vdict = {}
    
    
    for b1 in balls:
        for i in range(balls.index(ball) + 1, len(balls)):
            sv_x = balls[i].x - b1.x
            sv_y = balls[i].y - b1.y
            distance = np.sqrt(sv_x**2 + sv_y**2)
            distance_soll = b1.radius + balls[i].radius
                
            if distance < distance_soll:
                while distance < distance_soll:
                    b1.x += sv_x  
                    b1.y += sv_y
                    
                    
    while run:
        draw(WIN, balls)
        pygame.time.Clock().tick(FPS)
        
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                run = 0
                break
            
        for ball in balls:
            ball.borderCollision()
            for i in range(balls.index(ball) + 1, len(balls)): 
                ball.ballCollision(balls[i])                
            ball.move()
            
        for i in range(len(balls)):
            v_ball = "v_ball"+str(i)
            vnorm = np.sqrt(balls[i].vxy[0]**2 + balls[i].vxy[1]**2)
            vdict[v_ball] = round(vnorm * balls[i].speed)
            
        
    print(vdict)
    for key in vdict:
        plt.bar(key, vdict[key], 0.1, color="blue")
        plt.show()

        
    pygame.quit()


if __name__ == '__main__':
    main()